/* File:		CallFunc.cpp
 * Author(s):	Michael Deardeuff <mr__goodbytes@hotmail.com>
 * Description:
 *		Implements CallFunctionByName() and AddFunction() methods
 * Notes:
 *		Copyright (C) 2000 Michael Deardeuff
 *		You may use the code in whatever you want as long as the copyright notice stays intact.
 * Usage:
 *		To use these to functions(methods if you prefer), you first add the function to the
 *		"Callable Function List" by calling AddFunction() with the name you wish to give the
 *		function, and the function itself. Then to call the function in your code, you invoke
 *		CallFunctionByName() with the name you gave the function, and any arguments you may wish.
 *		to pass to the function.
 *		Example:
 *
 *			void CallMeFunc(void *args)
 *			{
 *				// Do Something...
 *			}
 *
 *			int main()
 *			{
 *				char funcCaller = "MyFunction";	// for later
 *				
 *				// Add the Function to the "Callable Function List"
 *				AddFunction("MyFunction", CallMeFunc);
 *				
 *				// Call the function in a variaty of ways
 *				CallFunctionByName("MyFunction", NULL);
 *				CallFunctionByName(funcCaller, NULL);
 *
 *				// ...
 *				return 0;
 *			}
 *		
 *		I find this useful when using a scripting language, or a console debugger in your game.
 *		Happy Coding!
 *		-Michael Deardeuff
 */

#ifdef _MSC_VER // >= 1200					// For MSVC++ Only (6.0+ in comments)
#pragma warning(disable :4786)				// Disable all those pesky warnings you get
#endif // _MSC_VER							// from using the STL

#include <map>

#include "mgTable.h"
#include "function.h"

// For use by our "map"
class FunctionPointer
{
public:
	void (*pFunc)(void *args);
};

StringTable functionTable;
std::map<char *, FunctionPointer> mFunction;

/* CallFunctionByName():
 *	Calls a function associated with to a string name
 *	Functions must be added to callable list via AddFunctions()
 */
bool CallFunctionByName (char * pName, void * args)
{
	char * result;

	// If the name is in the list, call it;
	if (functionTable.IsIn(pName))
	{
		result = (char*)functionTable.Get(pName);
		mFunction[ result ].pFunc( args );			// this is a good thing about maps
		return true;	//Ok
	}
	return false;	// Didn't happen
}

/* AddFunction():
 *	Adds functions and their associated strings to the list
 *	callable by CallFunctionName
 *	returns true if successful
 */
bool AddFunction(char * pName, void (*pFunc)(void *args))
{
	FunctionPointer fp;
	char * result;

	// Test to see whether pFunc is valid, and whether
	// pName is already listed in the set
	if ((pFunc != NULL) && (functionTable.IsIn(pName)))
		return false;	// Already something with that name in list

	// Puts name in the function table
	result = (char *) functionTable.Get(pName);
	
	// Assign the function pointer to the map
	// with the above result as the key
	fp.pFunc = pFunc;
	mFunction[result] = fp;
	
	return true;	// Ok
}